==> net.bluemind.p2.platform-5.4.2760.zip: net/bluemind/dataprotect/addressbook/impl/AddressBookBackupWorker.java <==
/* BEGIN LICENSE
  * Copyright © Blue Mind SAS, 2012-2024
  *
  * This file is part of Blue Mind. Blue Mind is a messaging and collaborative
  * solution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of either the GNU Affero General Public License as
  * published by the Free Software Foundation (version 3 of the License)
  * or the CeCILL as published by CeCILL.info (version 2 of the License).
  *
  * There are special exceptions to the terms and conditions of the
  * licenses as they are applied to this program. See LICENSE.txt in
  * the directory of this program distribution.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
  * See LICENSE.txt
  * END LICENSE
  */
package net.bluemind.dataprotect.addressbook.impl;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.vertx.core.buffer.Buffer;
import io.vertx.core.streams.ReadStream;
import net.bluemind.addressbook.api.IAddressBookUids;
import net.bluemind.addressbook.service.IInCoreIVCardService;
import net.bluemind.core.api.ListResult;
import net.bluemind.core.api.Stream;
import net.bluemind.core.api.fault.ServerFault;
import net.bluemind.core.container.api.ContainerQuery;
import net.bluemind.core.container.api.IContainers;
import net.bluemind.core.container.model.ContainerDescriptor;
import net.bluemind.core.container.model.ItemValue;
import net.bluemind.core.rest.vertx.VertxStream;
import net.bluemind.dataprotect.addressbook.impl.AddressBookBackupRepository.BackupUserAddressbookRecord;
import net.bluemind.dataprotect.api.IBackupWorker;
import net.bluemind.dataprotect.api.IDPContext;
import net.bluemind.dataprotect.api.PartGeneration;
import net.bluemind.dataprotect.api.WorkerDataType;
import net.bluemind.dataprotect.common.backup.RepositoryBackupPath;
import net.bluemind.dataprotect.service.internal.CommonBackupWorker;
import net.bluemind.dataprotect.service.internal.MailboxIndexJson;
import net.bluemind.dataprotect.service.tool.CoreCommonBackupWorker;
import net.bluemind.dataprotect.service.tool.ZipBuilder;
import net.bluemind.directory.api.BaseDirEntry;
import net.bluemind.directory.api.DirEntryQuery;
import net.bluemind.directory.api.IDirectory;
import net.bluemind.domain.api.Domain;
import net.bluemind.domain.api.IDomains;
import net.bluemind.domain.api.IInCoreDomains;
import net.bluemind.lib.vertx.VertxPlatform;
import net.bluemind.lib.vertx.utils.MmapWriteStream;
import net.bluemind.server.api.Server;
import net.bluemind.user.api.IUser;
import net.bluemind.user.api.User;

public class AddressBookBackupWorker implements IBackupWorker {
	private static Logger logger = LoggerFactory.getLogger(AddressBookBackupWorker.class);
	public static final Path WORKING_PATH = ZipBuilder.WORKING_PATH.resolve("addressbooks");
	private static final String ZIP_FILE_PREFIX = IAddressBookUids.TYPE;

	private static class AddressBookBackup extends CommonBackupWorker {
		private AddressBookBackupRepository repository;

		public AddressBookBackup(IDPContext ctx, Path p, PartGeneration partGen) {
			super(ctx, partGen);
			repository = new AddressBookBackupRepository(p);
		}

		@Override
		public List<MailboxIndexJson> partitionTreatment(String domainUid, String zipFile, List<String> partition) {
			String zipFileName = CommonBackupWorker.zipFilename(ZIP_FILE_PREFIX, zipFile);
			List<MailboxIndexJson> indexJson = new ArrayList<>();
			partition.forEach(uid -> {
				try {
					List<String> files = repository.writeUserAddressbooks(backupAddressBookUser(uid));
					indexJson.add(new MailboxIndexJson(files, uid, domainUid, zipFileName));
				} catch (IOException e) {
					logError(e, "Unable to backup user addressbooks {}", uid);
				}
			});
			indexJson.forEach(i -> i.zipFileName(zipFileName));
			indexJson.forEach(i -> i.domainUid(domainUid));
			return indexJson;
		}

		private MailboxIndexJsonList backupUsersAddressBooks(ItemValue<Domain> domain) {
			var q = DirEntryQuery.allWithHidden();
			q.kindsFilter = Arrays.asList(BaseDirEntry.Kind.USER);
			ListResult<String> searchUids = provider().instance(IDirectory.class, domain.uid).searchUids(q);
			return runIt(domain.uid, searchUids.values);
		}

		private List<BackupUserAddressbookRecord> backupAddressBookUser(String entryUid) {
			List<BackupUserAddressbookRecord> usersToBackup = new ArrayList<>();
			try {
				provider().instance(IContainers.class) //
						.all(ContainerQuery.ownerAndType(entryUid, IAddressBookUids.TYPE)) //
						.forEach(cd -> usersToBackup.add(backupUserAddressBook(cd)));
			} catch (Exception e) {
				logError(e, "Error trying to backup addressbooks for {}", entryUid);
			}

			return usersToBackup;
		}

		private BackupUserAddressbookRecord backupUserAddressBook(ContainerDescriptor containerDescriptor) {
			logger.info("Backup user addressbook {}", containerDescriptor.uid);
			IInCoreIVCardService vcardApi = provider().instance(IInCoreIVCardService.class, containerDescriptor.uid);
			IUser userApi = provider().instance(IUser.class, containerDescriptor.domainUid);
			ItemValue<User> user = userApi.getComplete(containerDescriptor.owner);
			ByteBuf out = readVcf(containerDescriptor, vcardApi);
			return new BackupUserAddressbookRecord(containerDescriptor, user, out);
		}

		private ByteBuf readVcf(ContainerDescriptor containerDescriptor, IInCoreIVCardService vcardApi) {
			ByteBuf out;
			try {
				MmapWriteStream target = new MmapWriteStream(AddressBookBackupWorker.WORKING_PATH, Integer.MAX_VALUE);
				Stream vcf = vcardApi.exportAllLight();
				ReadStream<Buffer> vcfStream = VertxStream.read(vcf);
				VertxPlatform.getVertx().executeBlocking(() -> vcfStream.pipeTo(target), false);
				out = target.mmap().orTimeout(5, TimeUnit.MINUTES).join();
			} catch (Exception e) {
				logger.warn("Unable to backup vcf data for addressbook {}: {}", containerDescriptor.uid,
						e.getMessage());
				out = Unpooled.EMPTY_BUFFER;
			}
			return out;
		}

		private void backupDomainAddressBook(String domainUid) {
			try {
				var q = DirEntryQuery.allWithHidden();
				q.kindsFilter = Arrays.asList(BaseDirEntry.Kind.ADDRESSBOOK);
				provider().instance(IDirectory.class, domainUid).searchUids(q).values.forEach(entryUid -> {
					provider().instance(IContainers.class) //
							.all(ContainerQuery.ownerAndType(entryUid, IAddressBookUids.TYPE)) //
							.forEach(containerDescriptor -> {
								try {
									logger.info("Backup domain addressbook {}", containerDescriptor.uid);
									IInCoreIVCardService vcardApi = provider().instance(IInCoreIVCardService.class,
											containerDescriptor.uid);
									IDomains domainApi = provider().instance(IDomains.class);
									ItemValue<Domain> domain = domainApi.get(containerDescriptor.domainUid);
									ByteBuf out = readVcf(containerDescriptor, vcardApi);
									repository.writeDomainAddressBook(containerDescriptor, domain, out);
								} catch (IOException e) {
									logError(e, "Unable to backup addressbook {}", containerDescriptor);
								}
							});
				});

			} catch (Exception e) {
				logError(e, "Error trying to backup addressbooks for {}", domainUid);
			}
		}

		public void backupAll() {
			var domainsApi = provider().instance(IInCoreDomains.class);
			List<MailboxIndexJson> domainsZip = new ArrayList<>();
			domainsApi.allUnfiltered().stream().filter(d -> !"global.virt".equals(d.uid)).forEach(domain -> {
				logger.info("Backup addressbooks for domain {}", domain.value.defaultAlias);
				backupDomainAddressBook(domain.uid);
				domainsZip.addAll(backupUsersAddressBooks(domain).mailboxesIndexInfo());
			});

			Path rootUserPath = repository.repositoryPath.resolveRootUserPath();
			try {
				RepositoryBackupPath.writeIndexFileToDir(rootUserPath, domainsZip);
			} catch (IOException e) {
				logError(e, "Error trying to write index.json {}", rootUserPath);
			}
		}

	}

	@Override
	public void prepareDataDirs(IDPContext ctx, PartGeneration partGen, ItemValue<Server> toBackup) throws ServerFault {
		cleanup(ctx, partGen, null);
		Path outputFolder = AddressBookBackupRepository.DEFAULT_PATH;
		long startDate = Instant.now().getEpochSecond();

		try {
			if (!Files.exists(WORKING_PATH)) {
				Files.createDirectories(WORKING_PATH);
			}

			logger.info("Starting addressbooks backup in {}", WORKING_PATH);
			ctx.info("Starting addressbooks backup");
			new AddressBookBackup(ctx, WORKING_PATH, partGen).backupAll();
			CoreCommonBackupWorker.prepareDataDirs(ctx, toBackup, WORKING_PATH, outputFolder);
		} catch (IOException e) {
			partGen.withErrors = true;
			ctx.error(e, "Unable to create temporary directory for addressbook backup");
		} finally {
			ctx.info(String.format("Ending addressbooks backup in %d seconds",
					Instant.now().getEpochSecond() - startDate));
		}
	}

	@Override
	public String getDataType() {
		return WorkerDataType.ADDRESSBOOK.value;
	}

	@Override
	public Set<String> getDataDirs() {
		return Set.of(AddressBookBackupRepository.DEFAULT_PATH.toString());
	}

	@Override
	public boolean supportsTag(String tag) {
		return CoreCommonBackupWorker.supportsTag(tag);
	}
}

==> net.bluemind.p2.platform-5.5.3112.zip: net/bluemind/dataprotect/addressbook/impl/AddressBookBackupWorker.java <==
/* BEGIN LICENSE
  * Copyright © Blue Mind SAS, 2012-2024
  *
  * This file is part of Blue Mind. Blue Mind is a messaging and collaborative
  * solution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of either the GNU Affero General Public License as
  * published by the Free Software Foundation (version 3 of the License)
  * or the CeCILL as published by CeCILL.info (version 2 of the License).
  *
  * There are special exceptions to the terms and conditions of the
  * licenses as they are applied to this program. See LICENSE.txt in
  * the directory of this program distribution.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
  * See LICENSE.txt
  * END LICENSE
  */
package net.bluemind.dataprotect.addressbook.impl;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.vertx.core.buffer.Buffer;
import io.vertx.core.streams.ReadStream;
import net.bluemind.addressbook.api.IAddressBookUids;
import net.bluemind.addressbook.service.IInCoreIVCardService;
import net.bluemind.core.api.ListResult;
import net.bluemind.core.api.Stream;
import net.bluemind.core.api.fault.ServerFault;
import net.bluemind.core.container.api.ContainerQuery;
import net.bluemind.core.container.api.IContainers;
import net.bluemind.core.container.model.ContainerDescriptor;
import net.bluemind.core.container.model.ItemValue;
import net.bluemind.core.rest.vertx.VertxStream;
import net.bluemind.dataprotect.addressbook.impl.AddressBookBackupRepository.BackupUserAddressbookRecord;
import net.bluemind.dataprotect.api.IBackupWorker;
import net.bluemind.dataprotect.api.IDPContext;
import net.bluemind.dataprotect.api.PartGeneration;
import net.bluemind.dataprotect.api.WorkerDataType;
import net.bluemind.dataprotect.common.backup.RepositoryBackupPath;
import net.bluemind.dataprotect.service.internal.CommonBackupWorker;
import net.bluemind.dataprotect.service.internal.MailboxIndexJson;
import net.bluemind.dataprotect.service.tool.CoreCommonBackupWorker;
import net.bluemind.dataprotect.service.tool.ZipBuilder;
import net.bluemind.directory.api.BaseDirEntry;
import net.bluemind.directory.api.BaseDirEntry.Kind;
import net.bluemind.directory.api.DirEntry;
import net.bluemind.directory.api.DirEntryQuery;
import net.bluemind.directory.api.IDirectory;
import net.bluemind.domain.api.Domain;
import net.bluemind.domain.api.IDomains;
import net.bluemind.domain.api.IInCoreDomains;
import net.bluemind.lib.vertx.VertxPlatform;
import net.bluemind.lib.vertx.utils.MmapWriteStream;
import net.bluemind.server.api.Server;
import net.bluemind.user.api.IUser;
import net.bluemind.user.api.User;

public class AddressBookBackupWorker implements IBackupWorker {
	private static Logger logger = LoggerFactory.getLogger(AddressBookBackupWorker.class);
	public static final Path WORKING_PATH = ZipBuilder.WORKING_PATH.resolve("addressbooks");
	private static final String ZIP_FILE_PREFIX = IAddressBookUids.TYPE;

	private static class AddressBookBackup extends CommonBackupWorker {
		private AddressBookBackupRepository repository;

		public AddressBookBackup(IDPContext ctx, Path p, PartGeneration partGen) {
			super(ctx, partGen);
			repository = new AddressBookBackupRepository(p);
		}

		@Override
		public List<MailboxIndexJson> partitionTreatment(String domainUid, String zipFile, List<String> partition) {
			IDirectory dirApi = provider().instance(IDirectory.class, domainUid);
			String zipFileName = CommonBackupWorker.zipFilename(ZIP_FILE_PREFIX, zipFile);
			List<MailboxIndexJson> indexJson = new ArrayList<>();
			partition.forEach(uid -> {
				try {
					DirEntry dirEntry = dirApi.findByEntryUid(uid);
					List<String> files = repository
							.writeUserAddressbooks(backupAddressBookUser(uid, dirEntry.kind));
					indexJson.add(new MailboxIndexJson(files, uid, domainUid, zipFileName));
				} catch (IOException e) {
					logError(e, "Unable to backup user addressbooks {}", uid);
				}
			});
			indexJson.forEach(i -> i.zipFileName(zipFileName));
			indexJson.forEach(i -> i.domainUid(domainUid));
			return indexJson;
		}

		private MailboxIndexJsonList backupUsersAddressBooks(ItemValue<Domain> domain) {
			var q = DirEntryQuery.allWithHidden();
			q.kindsFilter = Arrays.asList(BaseDirEntry.Kind.USER, BaseDirEntry.Kind.SHARED_MAILBOX);
			ListResult<String> searchUids = provider().instance(IDirectory.class, domain.uid).searchUids(q);
			return runIt(domain.uid, searchUids.values);
		}

		private List<BackupUserAddressbookRecord> backupAddressBookUser(String entryUid, Kind kind) {
			List<BackupUserAddressbookRecord> usersToBackup = new ArrayList<>();
			try {
				provider().instance(IContainers.class) //
						.all(ContainerQuery.ownerAndType(entryUid, IAddressBookUids.TYPE)) //
						.forEach(cd -> usersToBackup.add(backupUserAddressBook(cd, kind)));
			} catch (Exception e) {
				logError(e, "Error trying to backup {} addressbooks for {}", kind, entryUid);
			}

			return usersToBackup;
		}

		private BackupUserAddressbookRecord backupUserAddressBook(ContainerDescriptor containerDescriptor, Kind kind) {
			logger.info("Backup {} addressbook {}", kind, containerDescriptor.uid);
			IInCoreIVCardService vcardApi = provider().instance(IInCoreIVCardService.class, containerDescriptor.uid);
			IUser userApi = provider().instance(IUser.class, containerDescriptor.domainUid);
			ItemValue<User> user = userApi.getComplete(containerDescriptor.owner);
			ByteBuf out = readVcf(containerDescriptor, vcardApi);
			return new BackupUserAddressbookRecord(containerDescriptor, user, out);
		}

		private ByteBuf readVcf(ContainerDescriptor containerDescriptor, IInCoreIVCardService vcardApi) {
			ByteBuf out;
			try {
				MmapWriteStream target = new MmapWriteStream(AddressBookBackupWorker.WORKING_PATH, Integer.MAX_VALUE);
				Stream vcf = vcardApi.exportAllLight();
				ReadStream<Buffer> vcfStream = VertxStream.read(vcf);
				VertxPlatform.getVertx().executeBlocking(() -> vcfStream.pipeTo(target), false);
				out = target.mmap().orTimeout(5, TimeUnit.MINUTES).join();
			} catch (Exception e) {
				logger.warn("Unable to backup vcf data for addressbook {}: {}", containerDescriptor.uid,
						e.getMessage());
				out = Unpooled.EMPTY_BUFFER;
			}
			return out;
		}

		private void backupDomainAddressBook(String domainUid) {
			try {
				var q = DirEntryQuery.allWithHidden();
				q.kindsFilter = Arrays.asList(BaseDirEntry.Kind.ADDRESSBOOK);
				provider().instance(IDirectory.class, domainUid).searchUids(q).values.forEach(entryUid -> {
					provider().instance(IContainers.class) //
							.all(ContainerQuery.ownerAndType(entryUid, IAddressBookUids.TYPE)) //
							.forEach(containerDescriptor -> {
								try {
									logger.info("Backup domain addressbook {}", containerDescriptor.uid);
									IInCoreIVCardService vcardApi = provider().instance(IInCoreIVCardService.class,
											containerDescriptor.uid);
									IDomains domainApi = provider().instance(IDomains.class);
									ItemValue<Domain> domain = domainApi.get(containerDescriptor.domainUid);
									ByteBuf out = readVcf(containerDescriptor, vcardApi);
									repository.writeDomainAddressBook(containerDescriptor, domain, out);
								} catch (IOException e) {
									logError(e, "Unable to backup addressbook {}", containerDescriptor);
								}
							});
				});

			} catch (Exception e) {
				logError(e, "Error trying to backup addressbooks for {}", domainUid);
			}
		}

		public void backupAll() {
			var domainsApi = provider().instance(IInCoreDomains.class);
			List<MailboxIndexJson> domainsZip = new ArrayList<>();
			domainsApi.allUnfiltered().stream().filter(d -> !"global.virt".equals(d.uid)).forEach(domain -> {
				logger.info("Backup addressbooks for domain {}", domain.value.defaultAlias);
				backupDomainAddressBook(domain.uid);
				domainsZip.addAll(backupUsersAddressBooks(domain).mailboxesIndexInfo());
			});

			Path rootUserPath = repository.repositoryPath.resolveRootUserPath();
			try {
				RepositoryBackupPath.writeIndexFileToDir(rootUserPath, domainsZip);
			} catch (IOException e) {
				logError(e, "Error trying to write index.json {}", rootUserPath);
			}
		}

	}

	@Override
	public void prepareDataDirs(IDPContext ctx, PartGeneration partGen, ItemValue<Server> toBackup) throws ServerFault {
		cleanup(ctx, partGen, null);
		Path outputFolder = AddressBookBackupRepository.DEFAULT_PATH;
		long startDate = Instant.now().getEpochSecond();

		try {
			if (!Files.exists(WORKING_PATH)) {
				Files.createDirectories(WORKING_PATH);
			}

			logger.info("Starting addressbooks backup in {}", WORKING_PATH);
			ctx.info("Starting addressbooks backup");
			new AddressBookBackup(ctx, WORKING_PATH, partGen).backupAll();
			CoreCommonBackupWorker.prepareDataDirs(ctx, toBackup, WORKING_PATH, outputFolder);
		} catch (IOException e) {
			partGen.withErrors = true;
			ctx.error(e, "Unable to create temporary directory for addressbook backup");
		} finally {
			ctx.info(String.format("Ending addressbooks backup in %d seconds",
					Instant.now().getEpochSecond() - startDate));
		}
	}

	@Override
	public String getDataType() {
		return WorkerDataType.ADDRESSBOOK.value;
	}

	@Override
	public Set<String> getDataDirs() {
		return Set.of(AddressBookBackupRepository.DEFAULT_PATH.toString());
	}

	@Override
	public boolean supportsTag(String tag) {
		return CoreCommonBackupWorker.supportsTag(tag);
	}
}
